
#ifndef DRIVERS_OSD_FLASH_H_
#define DRIVERS_OSD_FLASH_H_

#include <linux/types.h>

/*---- read / wrire --------------------------------------------------------*/
#define FLASH_DRV_DEVNAME "osd_flash_spi"
/*---- IOCTL ---------------------------------------------------------------*/
#define FLASH_DRV_IOC_MAGIC  'i'
#define FLASH_IOCTL_MAGIC     0
/*---- Chip Command --------------------------------------------------------*/
#define FLASH_READ_JEDEC_ID              0x9F
#define FLASH_READ_STATUS1               0x05
#define FLASH_READ_STATUS2               0x35
#define FLASH_READ_STATUS3               0x15
#define FLASH_WRITE_STATUS1              0x01
#define FLASH_WRITE_STATUS2              0x31
#define FLASH_WRITE_STATUS3              0x11

#define FLASH_SET_WRITE_ENABLE           0x06
#define FLASH_SET_WRITE_DISABLE          0x04
#define FLASH_SET_SR_WRITE_ENABLE        0x50
#define FLASH_SET_ENABLE_RESET           0x66

#define FLASH_SET_PAGE_PROGRAM           0x02

#define FLASH_READ_SECURITY_REGISTER     0x5A
#define FLASH_PROGRAM_SECURITY_REGISTER  0x42
#define FLASH_ERASE_SECURITY_REGISTER    0x44

#define FLASH_SECTOR_ERASE               0x20
#define FLASH_BLOCK32_ERASE              0x52
#define FLASH_BLOCK64_ERASE              0xD8
#define FLASH_CHIP_ERASE                 0xC7

#define FLASH_NORMAL_READ_DATA           0x03
#define FLASH_FAST_READ_DATA             0x0B

/* Wait Time (usec) */
#define WAIT_READY_TIME                  500

/* Page Program Timeout(Max Sepc 3ms) */
#define PAGE_PROGRAM_TIMEOUT_MS         30		/* 30ms */
#define PAGE_PROGRAM_TIMEOUT            ((PAGE_PROGRAM_TIMEOUT_MS * 1000) / WAIT_READY_TIME)

/* Sector Erase(4KB) Timeout(Max Sepc 400ms) */
#define SECTOR_ERASE_TIMEOUT_MS         800	/* 800ms */
#define SECTOR_ERASE_TIMEOUT            ((SECTOR_ERASE_TIMEOUT_MS * 1000) / WAIT_READY_TIME)

/* Block Erase(32KB) Timeout(Max Sepc 1600ms) */
/* Block Erase(64KB) Timeout(Max Sepc 2000ms) */
#define BLOCK_ERASE_TIMEOUT_MS          2500	/* 2000ms */
#define BLOCK_ERASE_TIMEOUT             ((BLOCK_ERASE_TIMEOUT_MS * 1000) / WAIT_READY_TIME)

/* Chip Erase Timeout(Max Sepc 200s) */
#define CHIP_ERASE_TIMEOUT_MS           210000	/* 210000ms */
#define CHIP_ERASE_TIMEOUT              ((CHIP_ERASE_TIMEOUT_MS * 1000) / WAIT_READY_TIME)

/* Common status */
#define STATUS_WIP                      0x01

/* Return Value */
#define FLASH_DRV_OK                    ( 0)
#define FLASH_DRV_NG                    (-1)
#define FLASH_DRV_ERR                   (-2)

#define FLASH_IOCTL_MAX_LEN              8  /* addd+data */
#define FLASH_RW_CMD_OFFSET_LEN          4  /* Comannd(1) + Offset(3) */
#define FLASH_RW_DATA_LEN               16  /* Comannd(1) + Offset(3) */
#define FLASH_RW_DATA_SIZE              (FLASH_RW_CMD_OFFSET_LEN + FLASH_RW_DATA_LEN)  /* len + data */

struct flash_intr_status {
	uint8_t IntrCause;
	uint8_t InfoFsType;
	uint8_t ChStatus[5];
};

struct flash_dev_ioctl_params {
	uint8_t cmd;
	uint8_t addr;
	uint8_t value;
	int32_t lvalue;
	struct flash_intr_status intr_status;
};

#endif	/* DRIVERS_OSD_FLASH_H_ */
